"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const upath_1 = __importDefault(require("upath"));
const fs_1 = __importDefault(require("fs"));
const gulp_1 = require("gulp");
const gulp_rename_1 = __importDefault(require("gulp-rename"));
const gulp_filter_1 = __importDefault(require("gulp-filter"));
const event_stream_1 = __importDefault(require("event-stream"));
const lodash_1 = require("lodash");
const ejs_1 = __importDefault(require("ejs"));
const chalk_1 = __importDefault(require("chalk"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const babel = __importStar(require("@babel/core"));
const taroUtils_1 = require("./taroUtils");
const nextUtils_1 = require("./nextUtils");
const scssUtils_1 = __importDefault(require("./scssUtils"));
const utils_1 = require("./utils");
const constants_1 = require("./constants");
const openBrowser_1 = __importDefault(require("./openBrowser"));
const isWindows = process.platform === 'win32';
const DEFAULT_ROUTER_CONFIG = {
    mode: 'browser',
    customRoutes: {}
};
const DEFAULT_POSTCSS_OPTIONS = ['autoprefixer', 'pxtransform', 'cssModules'];
const DEFAULT_AUTOPREFIXER_OPTION = {
    enable: true,
    config: {
        flexbox: 'no-2009'
    }
};
const DEFAULT_PORT = '10086';
exports.default = (ctx, pluginOpts) => {
    const { paths, helper, runOpts } = ctx;
    const { appPath, outputPath, sourcePath, configPath } = paths;
    const runNextjs = pluginOpts.runNextjs == null && true;
    const browser = pluginOpts.browser == null && true;
    ctx.registerCommand({
        name: 'start',
        optionsMap: {
            '-p, --port': 'A port number on which to start the application'
        },
        synopsisList: [
            'taro start -p <port>'
        ],
        fn() {
            const port = runOpts.port || runOpts.p || DEFAULT_PORT;
            cross_spawn_1.default.sync('next', ['start', outputPath, '-p', port], {
                stdio: 'inherit'
            });
        }
    });
    ctx.registerPlatform({
        name: 'nextjs',
        useConfigName: 'h5',
        fn({ config }) {
            return __awaiter(this, void 0, void 0, function* () {
                const { router = DEFAULT_ROUTER_CONFIG, env, defineConstants, mode, alias = {}, sass = {}, designWidth = 750, postcss = {}, isWatch, devServer = {} } = config;
                if (router.mode !== 'browser') {
                    throw new Error('Next.js only support `browser` router mode.');
                }
                const appConfigFilePath = (0, utils_1.resolveScriptPath)(upath_1.default.join(sourcePath, `${helper.ENTRY}.config`));
                const appConfig = helper.readConfig(appConfigFilePath);
                const appFilePath = (0, utils_1.resolveScriptPath)(upath_1.default.join(sourcePath, helper.ENTRY));
                const outputSourcePath = upath_1.default.join(outputPath, 'src');
                const outputAppFilePath = upath_1.default.join(outputSourcePath, helper.ENTRY) + upath_1.default.extname(appFilePath);
                const nextAppFilePath = upath_1.default.join(outputPath, 'pages/_app.jsx');
                const templateDir = upath_1.default.resolve(__dirname, '../template');
                const taroPages = (0, taroUtils_1.mergeTaroPages)(appConfig.pages, appConfig.subPackages);
                const customRoutes = Object.create(null);
                if (router.customRoutes) {
                    for (const key of Object.keys(router.customRoutes)) {
                        customRoutes[(0, utils_1.ensureLeadingSlash)(key)] = router.customRoutes[key];
                    }
                }
                const additionalData = yield (0, scssUtils_1.default)(sass);
                const nextPageInfos = yield (0, nextUtils_1.getNextPageInfos)(sourcePath, taroPages, customRoutes);
                function createNextjsPages() {
                    const nextjsPagesDir = `${outputPath}/pages`;
                    for (const pageInfo of nextPageInfos) {
                        const nextRoute = pageInfo.route;
                        const targetPageFile = (function () {
                            if (nextRoute === '' || nextRoute === '/') {
                                return 'index.js';
                            }
                            const parts = nextRoute.split('/');
                            // 如果路由以 index 结尾，需要再嵌套一层目录
                            if (parts[parts.length - 1] === 'index') {
                                return `${nextRoute}/index.js`;
                            }
                            return `${nextRoute}.js`;
                        })();
                        const nextjsPageFilePath = upath_1.default.join(nextjsPagesDir, targetPageFile);
                        const nextjsPageDir = upath_1.default.dirname(nextjsPageFilePath);
                        if (!fs_1.default.existsSync(nextjsPageDir)) {
                            fs_1.default.mkdirSync(nextjsPageDir, { recursive: true });
                        }
                        const exportedFunctions = (0, nextUtils_1.getNextExportedFunctions)(upath_1.default.join(sourcePath, pageInfo.file));
                        const request = upath_1.default.join(outputSourcePath, pageInfo.request);
                        const modulePath = upath_1.default.relative(nextjsPageDir, request);
                        const configAbsolutePath = helper.resolveMainFilePath(`${request}.config`);
                        let contents;
                        if (fs_1.default.existsSync(configAbsolutePath)) {
                            // 读取 Taro 页面配置
                            // 注意：需要处理 definePageConfig 宏函数
                            const res = babel.transformFileSync(configAbsolutePath, {
                                presets: [['@babel/preset-env']],
                                plugins: ['@babel/plugin-proposal-class-properties']
                            });
                            const macro = 'function definePageConfig(config) { return config }\n';
                            const code = macro + res.code;
                            const pageConfig = eval(code);
                            contents = (0, utils_1.unIndent) `
                            import {TaroPage} from 'tarojs-plugin-platform-nextjs/taro'
                            import Page from '${modulePath}'
                            
                            const pageConfig = {
                                backgroundColor: ${JSON.stringify(pageConfig.backgroundColor)}
                            }

                            export default function NextPage(props) {
                                return <TaroPage {...props} Page={Page} pageConfig={pageConfig} />
                            }
                        `;
                        }
                        else {
                            contents = (0, utils_1.unIndent) `
                            import {TaroPage} from 'tarojs-plugin-platform-nextjs/taro'
                            import Page from '${modulePath}'

                            export default function NextPage(props) {
                                return <TaroPage {...props} Page={Page} />
                            }
                        `;
                        }
                        if (exportedFunctions.length) {
                            contents += `\nexport {${exportedFunctions.join(', ')}} from '${modulePath}'`;
                        }
                        fs_1.default.writeFileSync(nextjsPageFilePath, contents, { encoding: 'utf-8' });
                    }
                    // 处理自定义路由
                    const customRoutesFilePath = upath_1.default.join(outputPath, 'customRoutes.json');
                    fs_1.default.writeFileSync(customRoutesFilePath, JSON.stringify(customRoutes, null, '  '), { encoding: 'utf-8' });
                    // 处理 next.js 错误处理页面
                    for (const errorPage of ['_error', '404', '500']) {
                        const pagePath = (0, utils_1.resolveScriptPath)(upath_1.default.join(sourcePath, errorPage));
                        if (fs_1.default.existsSync(pagePath)) {
                            const modulePath = upath_1.default.normalize(upath_1.default.relative(nextjsPagesDir, upath_1.default.join(outputSourcePath, errorPage)));
                            const contents = `export {default} from '${modulePath}'`;
                            const nextjsErrorPagePath = upath_1.default.join(nextjsPagesDir, `${errorPage}.js`);
                            fs_1.default.writeFileSync(nextjsErrorPagePath, contents, { encoding: 'utf-8' });
                        }
                    }
                }
                createNextjsPages();
                function scaffold() {
                    return event_stream_1.default.merge((0, gulp_1.src)(`${appPath}/*.d.ts`).pipe((0, gulp_1.dest)(outputPath)), (0, gulp_1.src)(`${sourcePath}/**`)
                        .pipe((0, gulp_filter_1.default)(file => {
                        const stat = fs_1.default.statSync(file.path);
                        if (stat.isDirectory()) {
                            return true;
                        }
                        const ext = upath_1.default.extname(file.path);
                        if (!constants_1.SCRIPT_EXTS.includes(ext)) {
                            return true;
                        }
                        const dir = upath_1.default.dirname(file.path);
                        const name = upath_1.default.basename(file.path, ext);
                        const specifiedFilePath = upath_1.default.join(dir, `${name}.h5${ext}`);
                        return !fs_1.default.existsSync(specifiedFilePath);
                    }))
                        .pipe((0, gulp_rename_1.default)(p => {
                        const secondaryExt = upath_1.default.extname(p.basename);
                        if (secondaryExt === '.h5') {
                            p.basename = upath_1.default.basename(p.basename, secondaryExt);
                        }
                    }))
                        .pipe((0, gulp_1.dest)(outputSourcePath)), (0, gulp_1.src)(`${templateDir}/pages/_app.ejs`)
                        .pipe(event_stream_1.default.through(function (data) {
                        const ejsData = {
                            designWidth
                        };
                        const result = ejs_1.default.render(data.contents.toString(), ejsData);
                        data.contents = Buffer.from(result);
                        this.emit('data', data);
                    }))
                        .pipe((0, gulp_rename_1.default)('_app.jsx'))
                        .pipe((0, gulp_1.dest)(upath_1.default.join(outputPath, 'pages'))), (0, gulp_1.src)(`${templateDir}/pages/_document.jsx`).pipe((0, gulp_1.dest)(upath_1.default.join(outputPath, 'pages'))), (0, gulp_1.src)(`${appPath}/config/**`).pipe((0, gulp_1.dest)(upath_1.default.join(outputPath, 'config'))), (0, gulp_1.src)(`${templateDir}/next.config.ejs`)
                        .pipe(event_stream_1.default.through(function (data) {
                        const dynamicPages = nextPageInfos
                            .filter(pageInfo => pageInfo.dynamic)
                            .map(pageInfo => pageInfo.route);
                        const rewrites = (0, nextUtils_1.resolveDynamicPagesToRewrites)(dynamicPages);
                        const customNextConfigPath = upath_1.default.join(upath_1.default.dirname(configPath), 'next.config.js');
                        let customNextConfig = null;
                        if (fs_1.default.existsSync(customNextConfigPath)) {
                            customNextConfig = upath_1.default.relative(outputPath, customNextConfigPath);
                        }
                        const multipleRoutePages = Object.values(customRoutes).filter(value => Array.isArray(value));
                        for (const multipleRoute of multipleRoutePages) {
                            for (const source of multipleRoute.slice(1)) {
                                rewrites.push({
                                    source: (0, utils_1.ensureLeadingSlash)(source),
                                    destination: (0, utils_1.ensureLeadingSlash)(multipleRoute[0])
                                });
                            }
                        }
                        const ejsData = {
                            env,
                            defineConstants,
                            additionalData,
                            rewrites,
                            customNextConfig
                        };
                        const result = ejs_1.default.render(data.contents.toString(), ejsData);
                        data.contents = Buffer.from(result);
                        this.emit('data', data);
                    }))
                        .pipe((0, gulp_rename_1.default)('next.config.js'))
                        .pipe((0, gulp_1.dest)(outputPath)), (0, gulp_1.src)(`${templateDir}/postcss.config.ejs`)
                        .pipe(event_stream_1.default.through(function (data) {
                        const plugins = Object.entries(postcss).reduce((result, info) => {
                            const [pluginName, pluginOption] = info;
                            if (!DEFAULT_POSTCSS_OPTIONS.includes(pluginName) &&
                                (pluginOption === null || pluginOption === void 0 ? void 0 : pluginOption.enable)) {
                                const isRelative = pluginName.startsWith('./') ||
                                    pluginName.startsWith('../') ||
                                    ((isWindows && pluginName.startsWith('.\\')) ||
                                        pluginName.startsWith('..\\'));
                                let request = pluginName;
                                if (isRelative) {
                                    const absolutePath = upath_1.default.join(appPath, pluginName);
                                    request = upath_1.default.relative(outputPath, absolutePath);
                                }
                                const plugin = {
                                    request,
                                    option: pluginOption
                                };
                                result.push(plugin);
                            }
                            return result;
                        }, []);
                        const autoprefixerOption = (0, lodash_1.merge)({}, DEFAULT_AUTOPREFIXER_OPTION, postcss.autoprefixer);
                        const ejsData = {
                            designWidth,
                            autoprefixerOption: autoprefixerOption.enable
                                ? JSON.stringify(autoprefixerOption.config)
                                : null,
                            plugins
                        };
                        const result = ejs_1.default.render(data.contents.toString(), ejsData);
                        data.contents = Buffer.from(result);
                        this.emit('data', data);
                    }))
                        .pipe((0, gulp_rename_1.default)('postcss.config.js'))
                        .pipe((0, gulp_1.dest)(outputPath)), (0, gulp_1.src)(`${templateDir}/babel.config.ejs`)
                        .pipe(event_stream_1.default.through(function (data) {
                        const ejsData = {
                            nextAppFilePath: JSON.stringify(upath_1.default.relative(outputPath, nextAppFilePath)),
                            outputAppFilePath: JSON.stringify(upath_1.default.relative(outputPath, outputAppFilePath)),
                            taroPages: JSON.stringify(taroPages)
                        };
                        const result = ejs_1.default.render(data.contents.toString(), ejsData);
                        data.contents = Buffer.from(result);
                        this.emit('data', data);
                    }))
                        .pipe((0, gulp_rename_1.default)('babel.config.js'))
                        .pipe((0, gulp_1.dest)(outputPath)), (0, gulp_1.src)(`${templateDir}/tsconfig.json`)
                        .pipe(event_stream_1.default.through(function (data) {
                        const taroTSConfigPath = upath_1.default.join(appPath, 'tsconfig.json');
                        if (fs_1.default.existsSync(taroTSConfigPath)) {
                            const taroTSConfig = (0, utils_1.parseJson)(taroTSConfigPath);
                            const templateTSConfig = (0, utils_1.parseJson)(data.path);
                            let mergedTSConfig = templateTSConfig;
                            const paths = (0, utils_1.resolveAliasToTSConfigPaths)(alias, taroTSConfigPath);
                            mergedTSConfig = (0, lodash_1.merge)(taroTSConfig, templateTSConfig, { compilerOptions: { paths } });
                            data.contents = Buffer.from(JSON.stringify(mergedTSConfig, null, '  '));
                        }
                        this.emit('data', data);
                    }))
                        .pipe((0, gulp_1.dest)(outputPath)));
                }
                scaffold().on('end', () => __awaiter(this, void 0, void 0, function* () {
                    console.log(chalk_1.default.green('Compiled to Next.js application successfully!\n'));
                    const port = devServer.port || DEFAULT_PORT;
                    if (runNextjs) {
                        const args = [];
                        if (mode === 'development') {
                            args.push('dev');
                            args.push('-p', port);
                            (0, cross_spawn_1.default)('next', args, {
                                cwd: outputPath,
                                stdio: 'inherit'
                            });
                        }
                        else {
                            args.push('build');
                            const next = (0, cross_spawn_1.default)('next', args, {
                                cwd: outputPath,
                                stdio: 'inherit'
                            });
                            next.on('close', code => {
                                if (code !== 0) {
                                    process.exit(code || 1);
                                }
                            });
                        }
                    }
                    if (isWatch) {
                        if (browser) {
                            let indexRoute = customRoutes[taroPages[0]] || taroPages[0];
                            if (Array.isArray(indexRoute)) {
                                indexRoute = indexRoute[0];
                            }
                            if (indexRoute) {
                                const nextConfigPath = upath_1.default.resolve(outputPath, 'next.config.js');
                                const { basePath } = yield Promise.resolve().then(() => __importStar(require(nextConfigPath)));
                                let url = [undefined, '', '/'].includes(basePath)
                                    ? `http://127.0.0.1:${port}${indexRoute}`
                                    : `http://127.0.0.1:${port}${basePath}${indexRoute}`;
                                if (url.endsWith('/')) {
                                    url = url.substring(0, url.length - 1);
                                }
                                (0, openBrowser_1.default)(url);
                            }
                        }
                        function hasSpecifiedFile(filePath) {
                            const dir = upath_1.default.dirname(filePath);
                            const ext = upath_1.default.extname(filePath);
                            const base = upath_1.default.basename(filePath, ext);
                            if (!fs_1.default.existsSync(dir)) {
                                return false;
                            }
                            const files = fs_1.default.readdirSync(dir);
                            return files.some(name => {
                                const ext = upath_1.default.extname(name);
                                if (!constants_1.SCRIPT_EXTS.includes(ext)) {
                                    return false;
                                }
                                const primaryBase = upath_1.default.basename(name, ext);
                                const secondaryExt = upath_1.default.extname(primaryBase);
                                if (secondaryExt !== '.h5') {
                                    return false;
                                }
                                const secondaryBase = upath_1.default.basename(primaryBase, secondaryExt);
                                return secondaryBase === base;
                            });
                        }
                        function getOutputFilePath(filePath) {
                            const relativePath = filePath.substring(appPath.length + 1);
                            const ext = upath_1.default.extname(filePath);
                            if (!constants_1.SCRIPT_EXTS.includes(ext)) {
                                return upath_1.default.join(outputPath, relativePath);
                            }
                            const base = upath_1.default.basename(relativePath, ext);
                            const secondaryExt = upath_1.default.extname(base);
                            if (secondaryExt === '.h5') {
                                return upath_1.default.join(outputPath, upath_1.default.dirname(relativePath), upath_1.default.basename(base, secondaryExt) + ext);
                            }
                            if (hasSpecifiedFile(filePath)) {
                                return null;
                            }
                            return upath_1.default.join(outputPath, relativePath);
                        }
                        function handleWatch(operation, filePath) {
                            const outputFilePath = getOutputFilePath(filePath);
                            if (!outputFilePath) {
                                return;
                            }
                            const relativePath = filePath.substring(appPath.length + 1);
                            if (['changed', 'added'].includes(operation)) {
                                const outputDir = upath_1.default.dirname(outputFilePath);
                                if (!fs_1.default.existsSync(outputDir)) {
                                    fs_1.default.mkdirSync(outputDir, { recursive: true });
                                }
                                fs_1.default.copyFileSync(filePath, outputFilePath);
                            }
                            if (operation === 'removed') {
                                if (fs_1.default.existsSync(outputFilePath)) {
                                    fs_1.default.rmSync(outputFilePath);
                                }
                                const dir = upath_1.default.dirname(filePath);
                                const ext = upath_1.default.extname(filePath);
                                const base = upath_1.default.basename(filePath, ext);
                                const secondaryExt = upath_1.default.extname(base);
                                if (secondaryExt) {
                                    const primaryFilePath = upath_1.default.join(dir, `${upath_1.default.basename(base, secondaryExt)}${ext}`);
                                    if (fs_1.default.existsSync(primaryFilePath)) {
                                        const outputDir = upath_1.default.dirname(outputFilePath);
                                        if (!fs_1.default.existsSync(outputDir)) {
                                            fs_1.default.mkdirSync(outputDir, { recursive: true });
                                        }
                                        fs_1.default.copyFileSync(primaryFilePath, outputFilePath);
                                    }
                                }
                                else if (hasSpecifiedFile(filePath)) {
                                    const h5FilePath = upath_1.default.join(dir, `${base}.h5${ext}`);
                                    if (fs_1.default.existsSync(h5FilePath)) {
                                        const outputDir = upath_1.default.dirname(outputFilePath);
                                        if (!fs_1.default.existsSync(outputDir)) {
                                            fs_1.default.mkdirSync(outputDir, { recursive: true });
                                        }
                                        fs_1.default.copyFileSync(h5FilePath, outputFilePath);
                                    }
                                }
                            }
                            console.log(`${chalk_1.default.green(`File was ${operation}`)} ${relativePath}`);
                        }
                        const watcher = (0, gulp_1.watch)(`${upath_1.default.normalize(sourcePath)}/**`, { delay: 200 });
                        watcher.on('ready', () => console.log(chalk_1.default.blue('\nWatching for file changes...\n')));
                        watcher.on('change', filePath => handleWatch('changed', filePath));
                        watcher.on('add', filePath => handleWatch('added', filePath));
                        watcher.on('unlink', filePath => handleWatch('removed', filePath));
                        process.on('exit', () => {
                            watcher.close();
                        });
                    }
                }));
            });
        }
    });
};
