"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNextPageInfos = exports.findDynamicRoutePages = void 0;
const fs_1 = require("fs");
const path = __importStar(require("path"));
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const recursiveReadDir_1 = require("./recursiveReadDir");
const isDynamicPage_1 = require("./isDynamicPage");
function findDynamicRoutePages(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        const pages = [];
        const result = yield fs_1.promises.readdir(dir, { withFileTypes: true });
        yield Promise.all(result.map((part) => __awaiter(this, void 0, void 0, function* () {
            const absolutePath = path.join(dir, part.name);
            if (part.isDirectory() && (0, isDynamicPage_1.isDynamicPage)(part.name)) {
                const result = yield (0, recursiveReadDir_1.recursiveReadDir)(absolutePath, new RegExp(`\\.(?:${constants_1.SCRIPT_EXTS.map(ext => ext.substring(1)).join('|')})$`));
                for (const routePage of result) {
                    pages.push(`/${part.name}${routePage}`);
                }
                return;
            }
            if (part.isFile()) {
                const ext = path.extname(part.name);
                if (constants_1.SCRIPT_EXTS.includes(ext) && (0, isDynamicPage_1.isDynamicPage)(path.basename(part.name, ext))) {
                    pages.push(`/${part.name}`);
                }
            }
        })));
        return pages;
    });
}
exports.findDynamicRoutePages = findDynamicRoutePages;
function getNextPageInfos(sourcePath, taroPages, customRoutes) {
    return __awaiter(this, void 0, void 0, function* () {
        const routePages = [];
        for (const taroPage of taroPages) {
            const taroPageFile = (0, utils_1.resolveScriptPath)(path.join(sourcePath, taroPage));
            const taroPageDir = path.dirname(taroPageFile);
            let taroRoute = customRoutes[taroPage] || taroPage;
            if (Array.isArray(taroRoute)) {
                taroRoute = taroRoute[0];
            }
            const dynamicRoutePages = yield findDynamicRoutePages(taroPageDir);
            if (dynamicRoutePages.length) {
                for (const page of dynamicRoutePages) {
                    const ext = path.extname(page);
                    const absolutePath = path.join(taroPageDir, page);
                    routePages.push({
                        origin: taroPage,
                        route: taroRoute + page.substring(0, page.length - ext.length),
                        file: absolutePath.replace(sourcePath, ''),
                        request: absolutePath.replace(sourcePath, ''),
                        dynamic: true
                    });
                }
            }
            else {
                routePages.push({
                    origin: taroPage,
                    route: taroRoute,
                    file: taroPageFile.replace(sourcePath, ''),
                    request: taroPage,
                    dynamic: false
                });
            }
        }
        return routePages;
    });
}
exports.getNextPageInfos = getNextPageInfos;
