"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.recursiveReadDir = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
/**
 * Recursively read directory
 * @param  {string} dir Directory to read
 * @param  {RegExp} filter Filter for the file name, only the name part is considered, not the full path
 * @param  {string[]=[]} arr This doesn't have to be provided, it's used for the recursion
 * @param  {string=dir`} rootDir Used to replace the initial path, only the relative path is left, it's faster than path.relative.
 * @returns Promise array holding all relative paths
 */
function recursiveReadDir(dir, filter, ignore, arr = [], rootDir = dir) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield fs_1.promises.readdir(dir, { withFileTypes: true });
        yield Promise.all(result.map((part) => __awaiter(this, void 0, void 0, function* () {
            const absolutePath = (0, path_1.join)(dir, part.name);
            if (ignore && ignore.test(part.name))
                return;
            // readdir does not follow symbolic links
            // if part is a symbolic link, follow it using stat
            let isDirectory = part.isDirectory();
            if (part.isSymbolicLink()) {
                const stats = yield fs_1.promises.stat(absolutePath);
                isDirectory = stats.isDirectory();
            }
            if (isDirectory) {
                yield recursiveReadDir(absolutePath, filter, ignore, arr, rootDir);
                return;
            }
            if (!filter.test(part.name)) {
                return;
            }
            arr.push(absolutePath.replace(rootDir, ''));
        })));
        return arr.sort();
    });
}
exports.recursiveReadDir = recursiveReadDir;
