"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveAliasToTSConfigPaths = exports.parseJson = exports.resolveScriptPath = exports.ensureLeadingSlash = exports.unIndent = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const typescript_1 = __importDefault(require("typescript"));
const constants_1 = require("./constants");
function unIndent(strings, ...values) {
    const text = String.raw(strings, ...values);
    const lines = text.replace(/^\n/u, '').replace(/\n\s*$/u, '').split('\n');
    const lineIndents = lines.filter(line => line.trim()).map(line => line.match(/ */u)[0].length);
    const minLineIndent = Math.min(...lineIndents);
    return lines.map(line => line.slice(minLineIndent)).join('\n');
}
exports.unIndent = unIndent;
/**
 * For a given page path, this function ensures that there is a leading slash.
 * If there is not a leading slash, one is added, otherwise it is noop.
 */
function ensureLeadingSlash(path) {
    if (path == null) {
        return '';
    }
    return path.startsWith('/') ? path : `/${path}`;
}
exports.ensureLeadingSlash = ensureLeadingSlash;
function resolveScriptPath(filePath, extArrs = constants_1.SCRIPT_EXTS) {
    const taroEnv = 'h5';
    for (let i = 0; i < extArrs.length; i++) {
        const item = extArrs[i];
        if (taroEnv) {
            if (fs.existsSync(`${filePath}.${taroEnv}${item}`)) {
                return `${filePath}.${taroEnv}${item}`;
            }
            if (fs.existsSync(`${filePath}${path.sep}index.${taroEnv}${item}`)) {
                return `${filePath}${path.sep}index.${taroEnv}${item}`;
            }
            if (fs.existsSync(`${filePath.replace(/\/index$/, `.${taroEnv}/index`)}${item}`)) {
                return `${filePath.replace(/\/index$/, `.${taroEnv}/index`)}${item}`;
            }
        }
        if (fs.existsSync(`${filePath}${item}`)) {
            return `${filePath}${item}`;
        }
        if (fs.existsSync(`${filePath}${path.sep}index${item}`)) {
            return `${filePath}${path.sep}index${item}`;
        }
    }
    return filePath;
}
exports.resolveScriptPath = resolveScriptPath;
function parseJson(filePath) {
    const sourceText = fs.readFileSync(filePath, 'utf-8');
    const jsonFile = typescript_1.default.parseJsonText(filePath, sourceText);
    const errors = [];
    const result = typescript_1.default.convertToObject(jsonFile, errors);
    return result;
}
exports.parseJson = parseJson;
function resolveAliasToTSConfigPaths(alias, tsconfigPath) {
    const { baseUrl = '.' } = parseJson(tsconfigPath).compilerOptions;
    const pathsBaseUrl = path.resolve(path.dirname(tsconfigPath), baseUrl);
    const paths = {};
    Object.keys(alias).forEach(item => {
        const key = item + '/*';
        const value = path.relative(pathsBaseUrl, alias[item]);
        paths[key] = [value + '/*'];
    });
    return paths;
}
exports.resolveAliasToTSConfigPaths = resolveAliasToTSConfigPaths;
