"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  shouldBeObject: true,
  findDOM: true,
  getParameterError: true,
  setTransform: true,
  serializeParams: true,
  limited: true,
  unsupported: true,
  weixinCorpSupport: true,
  isFunction: true,
  isValidColor: true,
  easeInOutCubic: true
};
exports.easeInOutCubic = easeInOutCubic;
exports.findDOM = findDOM;
exports.getParameterError = getParameterError;
exports.isFunction = isFunction;
exports.limited = exports.isValidColor = void 0;
exports.serializeParams = serializeParams;
exports.setTransform = setTransform;
exports.shouldBeObject = shouldBeObject;
exports.unsupported = void 0;
exports.weixinCorpSupport = weixinCorpSupport;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _env = require("./env");

Object.keys(_env).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _env[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _env[key];
    }
  });
});

function shouldBeObject(target) {
  if (target && (0, _typeof2.default)(target) === 'object') {
    return {
      flag: true
    };
  }

  return {
    flag: false,
    msg: getParameterError({
      correct: 'Object',
      wrong: target
    })
  };
}

function findDOM(inst) {
  if (inst) {
    // eslint-disable-next-line react/no-find-dom-node
    return _reactDom.default.findDOMNode(inst);
  }

  return document.body;
}

function getParameterError(_ref) {
  var _ref$name = _ref.name,
      name = _ref$name === void 0 ? '' : _ref$name,
      para = _ref.para,
      correct = _ref.correct,
      wrong = _ref.wrong;
  var parameter = para ? "parameter.".concat(para) : 'parameter';
  var errorType = upperCaseFirstLetter(wrong === null ? 'Null' : (0, _typeof2.default)(wrong));

  if (name) {
    return "".concat(name, ":fail parameter error: ").concat(parameter, " should be ").concat(correct, " instead of ").concat(errorType);
  } else {
    return "parameter error: ".concat(parameter, " should be ").concat(correct, " instead of ").concat(errorType);
  }
}

function upperCaseFirstLetter(str) {
  return str.replace(/^./, function (match) {
    return match.toUpperCase();
  });
}

function setTransform(el, val) {
  el.style.webkitTransform = val;
  el.style.transform = val;
}

function serializeParams(params) {
  if (!params) {
    return '';
  }

  return Object.keys(params).map(function (key) {
    return "".concat(encodeURIComponent(key), "=").concat((0, _typeof2.default)(params[key]) === 'object' ? encodeURIComponent(JSON.stringify(params[key])) : encodeURIComponent(params[key]));
  }).join('&');
}
/**
 * 显式标识不能在服务端执行的 API。
 */


var limited;
/**
 * 显式处理不支持的 API。
 */

exports.limited = limited;

(function (_limited) {
  function never(api, fn) {
    var res = function res() {
      if (typeof window === 'undefined') {
        throw new Error("`".concat(api, "` cannot be called on server-side!"));
      }

      return fn.apply(void 0, arguments);
    };

    return res;
  }

  _limited.never = never;

  function _void(api, fn) {
    var res = function res() {
      if (typeof window === 'undefined') {
        console.error("`".concat(api, "` does nothing on the server-side!"));
      }

      return fn.apply(void 0, arguments);
    };

    return res;
  }

  _limited._void = _void;

  function async(api, fn) {
    var res = function res(option) {
      if (typeof window === 'undefined') {
        var fail = option.fail,
            complete = option.complete;
        var err = {
          errMsg: "`".concat(api, "` cannot be called on server-side!")
        };
        fail === null || fail === void 0 ? void 0 : fail(err);
        complete === null || complete === void 0 ? void 0 : complete();
        return;
      }

      return fn(option);
    };

    return res;
  }

  _limited.async = async;
})(limited || (exports.limited = limited = {}));

var unsupported;
exports.unsupported = unsupported;

(function (_unsupported) {
  function never(api) {
    return function () {
      throw new Error("Taro H5 \u7AEF\u4E0D\u652F\u6301 API - ".concat(api));
    };
  }

  _unsupported.never = never;

  function _void(api) {
    return function () {
      console.error("Taro H5 \u7AEF\u4E0D\u652F\u6301 API - ".concat(api));
    };
  }

  _unsupported._void = _void;

  function async(api) {
    return function () {
      var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          fail = _ref2.fail,
          complete = _ref2.complete;

      var err = {
        errMsg: "Taro H5 \u7AEF\u4E0D\u652F\u6301 API - ".concat(api)
      };
      fail === null || fail === void 0 ? void 0 : fail(err);
      complete === null || complete === void 0 ? void 0 : complete();
      return Promise.reject(err);
    };
  }

  _unsupported.async = async;
})(unsupported || (exports.unsupported = unsupported = {}));

function weixinCorpSupport(apiName) {
  return function () {
    var errMsg = "h5\u7AEF\u4EC5\u5728\u5FAE\u4FE1\u516C\u4F17\u53F7\u4E2D\u652F\u6301 API ".concat(apiName);

    if (process.env.NODE_ENV !== 'production') {
      console.error(errMsg);
      return Promise.reject({
        errMsg: errMsg
      });
    } else {
      console.warn(errMsg);
      return Promise.resolve({
        errMsg: errMsg
      });
    }
  };
}

function isFunction(obj) {
  return typeof obj === 'function';
}

var VALID_COLOR_REG = /^#[0-9a-fA-F]{6}$/;

var isValidColor = function isValidColor(color) {
  return VALID_COLOR_REG.test(color);
};

exports.isValidColor = isValidColor;

function easeInOutCubic(t, b, c, d) {
  var cc = c - b;
  t /= d / 2;

  if (t < 1) {
    return cc / 2 * t * t * t + b;
  }

  return cc / 2 * ((t -= 2) * t * t + 2) + b;
}