"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stopAccelerometer = exports.startAccelerometer = exports.onAccelerometerChange = exports.offAccelerometerChange = void 0;

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _util = require("../_util");

var _handler = require("../_util/handler");

var callbackManager = new _handler.CallbackManager();
var devicemotionListener;

var stopAccelerometerInternal = function stopAccelerometerInternal(_ref) {
  var success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;

  try {
    window.removeEventListener('devicemotion', devicemotionListener, true);
    success === null || success === void 0 ? void 0 : success();
  } catch (err) {
    fail === null || fail === void 0 ? void 0 : fail(err);
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 停止监听加速度数据。
 */


var stopAccelerometer = (0, _mpromisify.default)(_util.limited.async('stopAccelerometer', stopAccelerometerInternal));
exports.stopAccelerometer = stopAccelerometer;
var INTERVAL_MAP = {
  game: {
    interval: 20,
    frequency: 50
  },
  ui: {
    interval: 60,
    frequency: 16.67
  },
  normal: {
    interval: 200,
    frequency: 5
  }
};

var getDevicemotionListener = function getDevicemotionListener(interval) {
  var lock;
  var timer;
  return function (evt) {
    if (lock) return;
    lock = true;
    timer && clearTimeout(timer);
    callbackManager.trigger({
      x: evt.acceleration.x || 0,
      y: evt.acceleration.y || 0,
      z: evt.acceleration.z || 0
    });
    timer = setTimeout(function () {
      lock = false;
    }, interval);
  };
};

var startAccelerometerInternal = function startAccelerometerInternal(_ref2) {
  var _ref2$interval = _ref2.interval,
      interval = _ref2$interval === void 0 ? 'normal' : _ref2$interval,
      success = _ref2.success,
      fail = _ref2.fail,
      complete = _ref2.complete;

  if (window.DeviceMotionEvent) {
    var err = {
      errMsg: 'accelerometer is not supported'
    };
    fail === null || fail === void 0 ? void 0 : fail(err);
  } else {
    try {
      var intervalObj = INTERVAL_MAP[interval];

      if (devicemotionListener) {
        stopAccelerometer();
      }

      devicemotionListener = getDevicemotionListener(intervalObj.interval);
      window.addEventListener('devicemotion', devicemotionListener, true);
      success === null || success === void 0 ? void 0 : success();
    } catch (err) {
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: err.message
      });
    }
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 开始监听加速度数据。
 */


var startAccelerometer = (0, _mpromisify.default)(_util.limited.async('startAccelerometer', startAccelerometerInternal));
/**
 * 监听加速度数据事件。频率根据 Taro.startAccelerometer() 的 interval 参数。可使用 Taro.stopAccelerometer() 停止监听。
 */

exports.startAccelerometer = startAccelerometer;

var onAccelerometerChange = function onAccelerometerChange(callback) {
  callbackManager.add(callback);
};
/**
 * 取消监听加速度数据事件，参数为空，则取消所有的事件监听
 */


exports.onAccelerometerChange = onAccelerometerChange;

var offAccelerometerChange = function offAccelerometerChange(callback) {
  callbackManager.remove(callback);
};

exports.offAccelerometerChange = offAccelerometerChange;