"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setClipboardData = exports.getClipboardData = void 0;

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _util = require("../_util");

var globalTextArea = null;

var setClipboardDataInternal = function setClipboardDataInternal(_ref) {
  var data = _ref.data,
      success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;

  // 使用 try/catch 尝试是否支持 navigator.clipboard，如果不支持会在浏览器中看到 DOMExceptions。
  try {
    navigator.clipboard.writeText(data).then(function () {
      success === null || success === void 0 ? void 0 : success();
    }).catch(function (err) {
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: err.message
      });
    }).finally(function () {
      complete === null || complete === void 0 ? void 0 : complete();
    });
    return;
  } catch (_unused) {} // 回退 textarea 和 execCommand 方案解决


  var activeElement = document.activeElement;

  if (!globalTextArea) {
    globalTextArea = document.createElement('textarea');
    globalTextArea.setAttribute('aria-hidden', 'true');
    var textArea = document.body.appendChild(globalTextArea);
    textArea.style.height = '1px';
    textArea.style.width = '1px';
    textArea.style.position = 'absolute';
  }

  globalTextArea.value = data;
  globalTextArea.focus();
  globalTextArea.select();
  document.execCommand('copy');

  if (activeElement instanceof HTMLElement) {
    activeElement.focus();
  }

  document.body.removeChild(globalTextArea);
  success === null || success === void 0 ? void 0 : success();
  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 设置系统剪贴板的内容
 */


var setClipboardData = (0, _mpromisify.default)(_util.limited.async('setClipboardData', setClipboardDataInternal));
exports.setClipboardData = setClipboardData;

var getClipboardDataInternal = function getClipboardDataInternal(_ref2) {
  var success = _ref2.success,
      complete = _ref2.complete;
  var res = {
    data: globalTextArea ? globalTextArea.value : ''
  };
  success === null || success === void 0 ? void 0 : success(res);
  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 获取系统剪贴板的内容
 */


var getClipboardData = (0, _mpromisify.default)(_util.limited.async('getClipboardData', getClipboardDataInternal));
exports.getClipboardData = getClipboardData;