"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stopCompass = exports.startCompass = exports.onCompassChange = exports.offCompassChange = void 0;

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _handler = require("../_util/handler");

var _util = require("../_util");

var callbackManager = new _handler.CallbackManager();
var compassListener;

var stopCompassInternal = function stopCompassInternal(_ref) {
  var success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;

  try {
    window.removeEventListener('deviceorientation', compassListener, true);
    success === null || success === void 0 ? void 0 : success();
  } catch (err) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: err.message
    });
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 停止监听罗盘数据
 */


var stopCompass = (0, _mpromisify.default)(_util.limited.async('stopCompass', stopCompassInternal));
exports.stopCompass = stopCompass;

var getDeviceOrientationListener = function getDeviceOrientationListener(interval) {
  var lock;
  var timer;
  return function (evt) {
    if (lock) return;
    lock = true;
    timer && clearTimeout(timer);
    callbackManager.trigger({
      direction: 360 - evt.alpha
    });
    timer = setTimeout(function () {
      lock = false;
    }, interval);
  };
};

var startCompassInternal = function startCompassInternal(_ref2) {
  var success = _ref2.success,
      fail = _ref2.fail,
      complete = _ref2.complete;

  if (!window.DeviceOrientationEvent) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'compass is not supported'
    });
  } else {
    try {
      if (compassListener) {
        stopCompass();
      }

      compassListener = getDeviceOrientationListener(200);
      window.addEventListener('deviceorientation', compassListener, true);
      success === null || success === void 0 ? void 0 : success();
    } catch (err) {
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: err.message
      });
    }
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 开始监听罗盘数据
 */


var startCompass = (0, _mpromisify.default)(_util.limited.async('startCompass', startCompassInternal));
/**
 * 监听罗盘数据变化事件。频率：5 次/秒，接口调用后会自动开始监听，可使用 wx.stopCompass 停止监听。
 */

exports.startCompass = startCompass;

var onCompassChange = function onCompassChange(callback) {
  callbackManager.add(callback);
};
/**
 * 取消监听罗盘数据变化事件，参数为空，则取消所有的事件监听。
 */


exports.onCompassChange = onCompassChange;

var offCompassChange = function offCompassChange(callback) {
  callbackManager.remove(callback);
};

exports.offCompassChange = offCompassChange;