"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stopDeviceMotionListening = exports.startDeviceMotionListening = exports.onDeviceMotionChange = exports.offDeviceMotionChange = void 0;

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _handler = require("../_util/handler");

var _util = require("../_util");

var callbackManager = new _handler.CallbackManager();
var deviceMotionListener;
var INTERVAL_MAP = {
  game: {
    interval: 20,
    frequency: 50
  },
  ui: {
    interval: 60,
    frequency: 16.67
  },
  normal: {
    interval: 200,
    frequency: 5
  }
};

var stopDeviceMotionListeningInternal = function stopDeviceMotionListeningInternal(_ref) {
  var success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;

  try {
    window.removeEventListener('deviceorientation', deviceMotionListener, true);
    success === null || success === void 0 ? void 0 : success();
  } catch (e) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: e.message
    });
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 停止监听设备方向的变化。
 */


var stopDeviceMotionListening = (0, _mpromisify.default)(_util.limited.async('stopDeviceMotionListening', stopDeviceMotionListeningInternal));
exports.stopDeviceMotionListening = stopDeviceMotionListening;

var getDeviceOrientationListener = function getDeviceOrientationListener(interval) {
  var lock;
  var timer;
  return function (evt) {
    if (lock) return;
    lock = true;
    timer && clearTimeout(timer);
    callbackManager.trigger({
      alpha: evt.alpha,
      beta: evt.beta,
      gamma: evt.gamma
    });
    timer = setTimeout(function () {
      lock = false;
    }, interval);
  };
};

var startDeviceMotionListeningInternal = function startDeviceMotionListeningInternal(_ref2) {
  var _ref2$interval = _ref2.interval,
      interval = _ref2$interval === void 0 ? 'normal' : _ref2$interval,
      success = _ref2.success,
      fail = _ref2.fail,
      complete = _ref2.complete;

  if (!window.DeviceOrientationEvent) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'deviceMotion is not supported'
    });
  } else {
    try {
      var intervalObj = INTERVAL_MAP[interval];

      if (window.DeviceOrientationEvent) {
        if (deviceMotionListener) {
          stopDeviceMotionListening();
        }

        deviceMotionListener = getDeviceOrientationListener(intervalObj.interval);
        window.addEventListener('deviceorientation', deviceMotionListener, true);
      } else {
        throw new Error('deviceMotion is not supported');
      }

      success === null || success === void 0 ? void 0 : success();
    } catch (err) {
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: err.message
      });
    }
  }

  complete === null || complete === void 0 ? void 0 : complete();
};
/**
 * 开始监听设备方向的变化。
 */


var startDeviceMotionListening = (0, _mpromisify.default)(_util.limited.async('startDeviceMotionListening', startDeviceMotionListeningInternal));
/**
 * 监听设备方向变化事件。
 */

exports.startDeviceMotionListening = startDeviceMotionListening;

var onDeviceMotionChange = function onDeviceMotionChange(callback) {
  callbackManager.add(callback);
};
/**
 * 取消监听设备方向变化事件，参数为空，则取消所有的事件监听。
 */


exports.onDeviceMotionChange = onDeviceMotionChange;

var offDeviceMotionChange = function offDeviceMotionChange(callback) {
  callbackManager.remove(callback);
};

exports.offDeviceMotionChange = offDeviceMotionChange;