"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.onNetworkWeakChange = exports.onNetworkStatusChange = exports.offNetworkWeakChange = exports.offNetworkStatusChange = exports.getNetworkType = exports.getLocalIPAddress = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _handler = require("../_util/handler");

var _util = require("../_util");

function getConnection() {
  // @ts-ignore
  return navigator.connection || navigator.mozConnection || navigator.webkitConnection || navigator.msConnection;
}

var getNetworkTypeInernal = function getNetworkTypeInernal(_ref) {
  var success = _ref.success,
      complete = _ref.complete;
  var connection = getConnection();
  var networkType = 'unknown'; // 浏览器不支持获取网络状态

  if (connection) {
    // Supports only the navigator.connection.type value which doesn't match the latest spec.
    // https://www.davidbcalhoun.com/2010/using-navigator-connection-android/
    if (!isNaN(Number(connection.type))) {
      switch (connection.type) {
        case connection.WIFI:
          networkType = 'wifi';
          break;

        case connection.CELL_3G:
          networkType = '3g';
          break;

        case connection.CELL_2G:
          networkType = '2g';
          break;

        default:
          // ETHERNET, UNKNOWN
          networkType = 'unknown';
          break;
      }
    } else if (connection.type) {
      networkType = connection.type; // Only supports the type value.
    } else if (connection.effectiveType) {
      networkType = connection.effectiveType;
    }
  }

  success === null || success === void 0 ? void 0 : success({
    networkType: networkType
  });
  complete === null || complete === void 0 ? void 0 : complete();
};

var getNetworkType = (0, _mpromisify.default)(getNetworkTypeInernal);
exports.getNetworkType = getNetworkType;
var networkStatusManager = new _handler.CallbackManager();

var networkStatusListener = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee() {
    var _yield$getNetworkType, networkType, isConnected, obj;

    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return getNetworkType();

          case 2:
            _yield$getNetworkType = _context.sent;
            networkType = _yield$getNetworkType.networkType;
            isConnected = networkType !== 'none';
            obj = {
              isConnected: isConnected,
              networkType: networkType
            };
            networkStatusManager.trigger(obj);

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function networkStatusListener() {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * 监听弱网状态变化事件。
 */


var onNetworkWeakChange = _util.unsupported._void('onNetworkWeakChange');

exports.onNetworkWeakChange = onNetworkWeakChange;

var onNetworkStatusChange = function onNetworkStatusChange(callback) {
  networkStatusManager.add(callback);
  var connection = getConnection();

  if (connection && networkStatusManager.count() === 1) {
    connection.addEventListener('change', networkStatusListener);
  }
};
/**
 * 取消监听弱网状态变化事件。
 */


exports.onNetworkStatusChange = onNetworkStatusChange;

var offNetworkWeakChange = _util.unsupported._void('offNetworkStatusChange');

exports.offNetworkWeakChange = offNetworkWeakChange;

var offNetworkStatusChange = function offNetworkStatusChange(callback) {
  networkStatusManager.remove(callback);
  var connection = getConnection();

  if (connection && networkStatusManager.count() === 0) {
    connection.removeEventListener('change', networkStatusListener);
  }
};
/**
 * 获取局域网 IP 地址
 */


exports.offNetworkStatusChange = offNetworkStatusChange;

var getLocalIPAddress = _util.unsupported.never('getLocalIPAddress');

exports.getLocalIPAddress = getLocalIPAddress;