"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SelectorQuery = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _CanvasContext = require("../canvas/CanvasContext");

var _util = require("../_util");

var _nodesRef = require("./nodesRef");

function filter(fields, dom, selector) {
  if (!dom) return null;
  var isViewport = selector === '.taro_page';
  var id = fields.id,
      dataset = fields.dataset,
      rect = fields.rect,
      size = fields.size,
      scrollOffset = fields.scrollOffset,
      _fields$properties = fields.properties,
      properties = _fields$properties === void 0 ? [] : _fields$properties,
      _fields$computedStyle = fields.computedStyle,
      computedStyle = _fields$computedStyle === void 0 ? [] : _fields$computedStyle,
      nodeCanvasType = fields.nodeCanvasType,
      node = fields.node,
      context = fields.context;
  var res = {};

  if (nodeCanvasType && node) {
    var tagName = dom.tagName;
    res.node = {
      id: dom.id,
      $taroElement: dom
    };

    if (/^taro-canvas-core/i.test(tagName)) {
      var type = dom.type || '';
      res.nodeCanvasType = type;
      var canvas = dom.getElementsByTagName('canvas')[0];

      if (/^(2d|webgl)/i.test(type) && canvas) {
        res.node = canvas;
      } else {
        res.node = null;
      }
    } else {
      // TODO https://developers.weixin.qq.com/miniprogram/dev/component/scroll-view.html
      // if (/^taro-scroll-view-core/i.test(tagName))
      res.nodeCanvasType = '';
      res.node = dom;
    }

    return res;
  }

  if (context) {
    var _tagName = dom.tagName;

    if (/^taro-video-core/i.test(_tagName)) {
      // TODO HTMLVideoElement to VideoContext
      return {
        context: dom
      };
    } else if (/^taro-canvas-core/i.test(_tagName)) {
      var _type = dom.type || '2d';

      var _canvas = dom === null || dom === void 0 ? void 0 : dom.querySelector('canvas');

      var ctx = _canvas === null || _canvas === void 0 ? void 0 : _canvas.getContext(_type);
      return {
        context: new _CanvasContext.CanvasContext(_canvas, ctx)
      };
    } else if (/^taro-live-player-core/i.test(_tagName)) {
      console.error('暂时不支持通过 NodesRef.context 获取 LivePlayerContext');
    } else if (/^taro-editor-core/i.test(_tagName)) {
      console.error('暂时不支持通过 NodesRef.context 获取 EditorContext');
    } else if (/^taro-map-core/i.test(_tagName)) {
      console.error('暂时不支持通过 NodesRef.context 获取 MapContext');
    }

    return;
  }

  if (id) res.id = dom.id;
  if (dataset) res.dataset = Object.assign({}, dom.dataset);

  if (rect || size) {
    var _dom$getBoundingClien = dom.getBoundingClientRect(),
        left = _dom$getBoundingClien.left,
        right = _dom$getBoundingClien.right,
        top = _dom$getBoundingClien.top,
        bottom = _dom$getBoundingClien.bottom,
        width = _dom$getBoundingClien.width,
        height = _dom$getBoundingClien.height;

    if (rect) {
      if (!isViewport) {
        res.left = left;
        res.right = right;
        res.top = top;
        res.bottom = bottom;
      } else {
        res.left = 0;
        res.right = 0;
        res.top = 0;
        res.bottom = 0;
      }
    }

    if (size) {
      if (!isViewport) {
        res.width = width;
        res.height = height;
      } else {
        res.width = dom.clientWidth;
        res.height = dom.clientHeight;
      }
    }
  }

  if (scrollOffset) {
    res.scrollLeft = dom.scrollLeft;
    res.scrollTop = dom.scrollTop;
  }

  if (properties.length) {
    properties.forEach(function (prop) {
      var attr = dom.getAttribute(prop);
      if (attr) res[prop] = attr;
    });
  }

  if (computedStyle.length) {
    var styles = window.getComputedStyle(dom);
    computedStyle.forEach(function (key) {
      var value = styles.getPropertyValue(key) || styles[key];
      if (value) res[key] = value;
    });
  }

  return res;
}
/**
 * WXML节点信息API
 * @return {Object} SelectorQuery 对象实例
 */


function queryBat(queue, cb) {
  var result = [];
  queue.forEach(function (item) {
    var selector = item.selector,
        single = item.single,
        fields = item.fields,
        component = item.component; // selector 的容器节点

    /* eslint-disable */

    var container = component !== null ? (0, _util.findDOM)(component) || document : document;
    /* eslint-enable */
    // 特殊处理 ---- 选自己

    var selectSelf = false;

    if (container !== document) {
      var _container$parentNode;

      var $nodeList = (_container$parentNode = container.parentNode) === null || _container$parentNode === void 0 ? void 0 : _container$parentNode.querySelectorAll(selector);

      if ($nodeList) {
        for (var i = 0, len = $nodeList.length; i < len; ++i) {
          if (container === $nodeList[i]) {
            selectSelf = true;
            break;
          }
        }
      }
    }

    if (single) {
      var el = selectSelf === true ? container : container.querySelector(selector);
      result.push(filter(fields, el, selector));
    } else {
      var $children = container.querySelectorAll(selector);
      var children = [];
      selectSelf === true && children.push(container);

      for (var _i = 0, _len = $children.length; _i < _len; ++_i) {
        children.push($children[_i]);
      }

      result.push(children.map(function (dom) {
        return filter(fields, dom);
      }));
    }
  });
  cb(result);
}

var SelectorQuery = /*#__PURE__*/function () {
  function SelectorQuery() {
    (0, _classCallCheck2.default)(this, SelectorQuery);
    this._defaultWebviewId = null;
    this._webviewId = null;
    this._queue = [];
    this._queueCb = [];
    this._component;
  }

  (0, _createClass2.default)(SelectorQuery, [{
    key: "in",
    value: function _in(component) {
      this._component = component;
      return this;
    }
  }, {
    key: "select",
    value: function select(selector) {
      // 小程序里跨自定义组件的后代选择器 '>>>' 在 h5 替换为普通后代选择器 '>'
      if (typeof selector === 'string') selector = selector.replace('>>>', '>');
      return new _nodesRef.NodesRef(selector, this, true);
    }
  }, {
    key: "selectAll",
    value: function selectAll(selector) {
      // 小程序里跨自定义组件的后代选择器 '>>>' 在 h5 替换为普通后代选择器 '>'
      if (typeof selector === 'string') selector = selector.replace('>>>', '>');
      return new _nodesRef.NodesRef(selector, this, false);
    }
  }, {
    key: "selectViewport",
    value: function selectViewport() {
      return new _nodesRef.NodesRef('.taro_page', this, true);
    }
  }, {
    key: "exec",
    value: function exec(cb) {
      var _this = this;

      queryBat(this._queue, function (res) {
        var _queueCb = _this._queueCb;
        res.forEach(function (item, index) {
          var cb = _queueCb[index];
          typeof cb === 'function' && cb.call(_this, item);
        });
        typeof cb === 'function' && cb.call(_this, res);
      });
      return this;
    }
  }, {
    key: "_push",
    value: function _push(selector, component, single, fields) {
      var callback = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;

      this._queue.push({
        component: component,
        selector: selector,
        single: single,
        fields: fields
      });

      this._queueCb.push(callback);
    }
  }]);
  return SelectorQuery;
}();

exports.SelectorQuery = SelectorQuery;