"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ENV_TYPE = void 0;
exports.getCurrentInstance = getCurrentInstance;
exports.getCurrentPages = getCurrentPages;
exports.getEnv = getEnv;
exports.initPxTransform = initPxTransform;
exports.nextTick = nextTick;
exports.pxTransform = pxTransform;

var _router = _interopRequireDefault(require("next/router"));

var _internal = require("./internal");

/**
 * 在类组件中，从属性中获取 Next.js 的路由对象。
 */
function getCurrentInstance(info) {
  var instance = {
    router: {}
  };

  if (!info) {
    if (typeof window === 'undefined') {
      throw new Error('`getCurrentInstance()` should be called in component scope!');
    }

    Object.defineProperty(instance.router, 'params', {
      get: function get() {
        var nextRouter = _router.default.router;

        if (!nextRouter) {
          throw new Error('Next.js router is not initialized!');
        }

        return Object.keys(nextRouter.query).reduce(function (result, key) {
          var value = nextRouter.query[key];

          if (typeof value === 'string') {
            result[key] = value;
          }

          return result;
        }, {});
      }
    });
    Object.defineProperty(instance.router, 'path', {
      get: function get() {
        var nextRouter = _router.default.router;

        if (!nextRouter) {
          throw new Error('Next.js router is not initialized!');
        }

        return nextRouter.pathname;
      }
    });
    return instance;
  }

  if (info.type === 'class') {
    if (!info.component) {
      throw new Error('`getCurrentInstance()` cannot get component instance!');
    }

    if (!info.component.props.router) {
      throw new Error('`getCurrentInstance()` cannot get Next.js router in props!');
    }

    Object.defineProperty(instance.router, 'params', {
      get: function get() {
        var nextRouter = info.component.props.router;

        if (!nextRouter) {
          throw new Error('Next.js router is not initialized!');
        }

        return Object.keys(nextRouter.query).reduce(function (result, key) {
          var value = nextRouter.query[key];

          if (typeof value === 'string') {
            result[key] = value;
          }

          return result;
        }, {});
      }
    });
    Object.defineProperty(instance.router, 'pathname', {
      get: function get() {
        var nextRouter = info.component.props.router;

        if (!nextRouter) {
          throw new Error('Next.js router is not initialized!');
        }

        return nextRouter.pathname;
      }
    });
    return instance;
  }

  throw new Error('`getCurrentInstance()` cannot called in function component currently!');
}

function getCurrentPages() {
  var taroApp = (0, _internal.getTaroApp)();
  return taroApp.getCurrentPages();
}

var ENV_TYPE;
exports.ENV_TYPE = ENV_TYPE;

(function (ENV_TYPE) {
  ENV_TYPE["WEB"] = "WEB";
})(ENV_TYPE || (exports.ENV_TYPE = ENV_TYPE = {}));

function getEnv() {
  return ENV_TYPE.WEB;
}

var designWidth;

function initPxTransform(options) {
  designWidth = options.designWidth;
}

function pxTransform(size) {
  if (designWidth == null) {
    throw new Error('`designWidth` is not initialized');
  }

  return Math.ceil(size / 40 * 640 / designWidth * 10000) / 10000 + 'rem';
}

function nextTick(callback) {
  setTimeout(callback);
}