"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useDidHide = useDidHide;
exports.useDidShow = useDidShow;
exports.usePageScroll = usePageScroll;
exports.useReachBottom = useReachBottom;
exports.useReady = useReady;
exports.useResize = useResize;
exports.useRouter = useRouter;
exports.useShareAppMessage = useShareAppMessage;
exports.useShareTimeline = useShareTimeline;

var _react = require("react");

var _router = require("next/router");

function useDepRef(prop) {
  var ref = (0, _react.useRef)(prop);
  (0, _react.useEffect)(function () {
    ref.current = prop;
  }, [prop]);
  return ref;
}

function useDidShow(callback) {
  var ref = useDepRef(callback);
  (0, _react.useEffect)(function () {
    function handleVisibilityChanged() {
      if (document.visibilityState === 'visible') {
        ref.current();
      }
    }

    document.addEventListener('visibilitychange', handleVisibilityChanged);
    return function () {
      document.removeEventListener('visibilitychange', handleVisibilityChanged);
    };
  }, []);
}

function useDidHide(callback) {
  var ref = useDepRef(callback);
  (0, _react.useEffect)(function () {
    function handleVisibilityChanged() {
      if (document.visibilityState !== 'visible') {
        ref.current();
      }
    }

    document.addEventListener('visibilitychange', handleVisibilityChanged);
    return function () {
      document.removeEventListener('visibilitychange', handleVisibilityChanged);
    };
  }, []);
} // TODO
// export function usePullDownRefresh(callback: () => void): void {
// }


function useReachBottom(callback) {
  var ref = useDepRef(callback);
  (0, _react.useEffect)(function () {
    function handleScroll() {
      var scrollHeight = Math.max(document.documentElement.scrollHeight, document.body.scrollHeight);
      var scrollTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
      var clientHeight = window.innerHeight || Math.min(document.documentElement.clientHeight, document.body.clientHeight);

      if (scrollTop >= scrollHeight - clientHeight - 60) {
        ref.current();
      }
    }

    window.addEventListener('scroll', handleScroll);
    return function () {
      window.removeEventListener('scroll', handleScroll);
    };
  }, []);
}

function useResize(callback) {
  var ref = useDepRef(callback);
  (0, _react.useEffect)(function () {
    function handleResized() {
      if (document.visibilityState !== 'visible') {
        ref.current();
      }
    }

    document.addEventListener('resize', handleResized);
    return function () {
      document.removeEventListener('resize', handleResized);
    };
  }, []);
}

function useReady(callback) {
  (0, _react.useEffect)(function () {
    callback();
  }, []);
}

function useRouter() {
  var nextRouter = (0, _router.useRouter)();
  var params = Object.keys(nextRouter.query).reduce(function (result, key) {
    var value = nextRouter.query[key];

    if (typeof value === 'string') {
      result[key] = value;
    }

    return result;
  }, {});
  var path = nextRouter.pathname;
  return {
    params: params,
    path: path
  };
}

function usePageScroll(callback) {
  var ref = useDepRef(callback);
  (0, _react.useEffect)(function () {
    function handleScroll() {
      var scrollingElement = document.scrollingElement || document.documentElement;
      var scrollTop = scrollingElement.scrollTop;
      ref.current({
        scrollTop: scrollTop
      });
    }

    document.addEventListener('scroll', handleScroll);
    return function () {
      document.removeEventListener('scroll', handleScroll);
    };
  }, []);
}

function useShareAppMessage() {}

function useShareTimeline() {}