"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getLocation = void 0;

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _util = require("./_util");

var getLocationInternal = function getLocationInternal(_ref) {
  var _ref$type = _ref.type,
      type = _ref$type === void 0 ? 'wgs84' : _ref$type,
      altitude = _ref.altitude,
      success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;
  var positionOptions = {
    enableHighAccuracy: !!altitude,
    // 海拔定位需要高精度
    timeout: 3000,
    maximumAge: 900000
  }; // Web端暂时仅支持GPS坐标系

  if (type.toUpperCase() !== 'WGS84') {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'This coordinate system type is not temporarily supported'
    });
    complete === null || complete === void 0 ? void 0 : complete();
    return;
  }

  if (!navigator.geolocation) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'getLocation is not support'
    });
    complete === null || complete === void 0 ? void 0 : complete();
    return;
  }

  navigator.geolocation.getCurrentPosition(function (position) {
    var _position$coords = position.coords,
        accuracy = _position$coords.accuracy,
        altitude = _position$coords.altitude,
        latitude = _position$coords.latitude,
        longitude = _position$coords.longitude,
        altitudeAccuracy = _position$coords.altitudeAccuracy;
    var result = {
      /** 位置的精确度 */
      accuracy: accuracy,

      /** 高度，单位 m */
      altitude: altitude || 0,

      /** 水平精度，单位 m */
      horizontalAccuracy: accuracy,

      /** 纬度，范围为 -90~90，负数表示南纬 */
      latitude: latitude,

      /** 经度，范围为 -180~180，负数表示西经 */
      longitude: longitude,

      /** 速度，单位 m/s */
      speed: position.coords.speed,

      /** 垂直精度，单位 m（Android 无法获取，返回 0） */
      verticalAccuracy: altitudeAccuracy || 0
    };
    success(result);
    complete === null || complete === void 0 ? void 0 : complete();
  }, function (err) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: err.message
    });
    complete === null || complete === void 0 ? void 0 : complete();
  }, positionOptions);
};

var getLocation = (0, _mpromisify.default)(_util.limited.async('getLocation', getLocationInternal));
exports.getLocation = getLocation;