"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.previewImage = exports.getImageInfo = exports.compressImage = exports.chooseImage = void 0;
exports.registerPreviewComponent = registerPreviewComponent;
exports.saveImageToPhotosAlbum = void 0;

var _createForOfIteratorHelper2 = _interopRequireDefault(require("@babel/runtime/helpers/createForOfIteratorHelper"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _util = require("../_util");

var _dom = require("../_util/dom");

var _jsxRuntime = require("react/jsx-runtime");

var chooseImageInternal = function chooseImageInternal(_ref) {
  var _ref$count = _ref.count,
      count = _ref$count === void 0 ? 1 : _ref$count,
      success = _ref.success,
      complete = _ref.complete,
      _ref$sourceType = _ref.sourceType,
      sourceType = _ref$sourceType === void 0 ? ['album', 'camera'] : _ref$sourceType;
  var el = (0, _dom.createInput)(sourceType);

  if (count > 1) {
    el.setAttribute('multiple', 'multiple');
  }

  el.setAttribute('accept', 'image/*');

  el.onchange = function (e) {
    var result = {
      tempFilePaths: [],
      tempFiles: []
    };
    var target = e.target;
    var files = target.files ? Array.from(target.files) : [];

    var _iterator = (0, _createForOfIteratorHelper2.default)(files),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var _result$tempFiles;

        var file = _step.value;
        var blob = new Blob([file], {
          type: file.type
        });
        var url = URL.createObjectURL(blob);
        result.tempFilePaths.push(url);
        (_result$tempFiles = result.tempFiles) === null || _result$tempFiles === void 0 ? void 0 : _result$tempFiles.push({
          path: url,
          size: file.size
        });
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    success(result);
    complete === null || complete === void 0 ? void 0 : complete();
  };

  var event = document.createEvent('MouseEvents');
  event.initEvent('click', true, true);
  el.dispatchEvent(event);
};
/**
 * 从本地相册选择图片或使用相机拍照。
 *
 * @example
 * ```javascript
 * Taro.chooseImage({
 *   count: 1, // 默认9
 *   sizeType: ['original', 'compressed'], // 可以指定是原图还是压缩图，默认二者都有
 *   sourceType: ['album', 'camera'], // 可以指定来源是相册还是相机，默认二者都有
 *   success: function (res) {
 *     // 返回选定照片的本地文件路径列表，tempFilePath可以作为img标签的src属性显示图片
 *     var tempFilePaths = res.tempFilePaths
 *   }
 * })
 * ```
 */


var chooseImage = (0, _mpromisify.default)(_util.limited.async('chooseImage', chooseImageInternal));
exports.chooseImage = chooseImage;
var Preview = null;
/**
 * 内部方法，注册预览组件
 */

function registerPreviewComponent(Target) {
  Preview = Target;
}

var previewContainer = null;

var previewImageInternal = function previewImageInternal(_ref2) {
  var current = _ref2.current,
      urls = _ref2.urls,
      success = _ref2.success,
      fail = _ref2.fail,
      complete = _ref2.complete;

  if (!Preview) {
    if (process.env.NODE_ENV === 'development') {
      console.error('`Preview` component is not registered.');
    }

    return;
  }

  if (previewContainer) {
    return;
  }

  if (!urls) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'The urls param is required.'
    });
    complete === null || complete === void 0 ? void 0 : complete();
    return;
  }

  previewContainer = document.createElement('div');
  document.body.appendChild(previewContainer);

  _reactDom.default.render( /*#__PURE__*/(0, _jsxRuntime.jsx)(Preview, {
    defaultCurrent: current,
    urls: urls,
    onClose: function onClose() {
      _reactDom.default.unmountComponentAtNode(previewContainer);

      previewContainer = null;
    }
  }), previewContainer);

  success === null || success === void 0 ? void 0 : success({});
};
/**
 * 在新页面中全屏预览图片。预览的过程中用户可以进行保存图片、发送给朋友等操作。
 *
 * @example
 * ```javascript
 * Taro.previewImage({
 *   current: '', // 当前显示图片的http链接
 *   urls: [] // 需要预览的图片http链接列表
 * })
 * ```
 */


var previewImage = (0, _mpromisify.default)(_util.limited.async('previewImage', previewImageInternal));
/**
 * 获取图片信息
 *
 * **示例代码：**
 *
 * ```javascript
 * Taro.getImageInfo({
 *   src: 'images/a.jpg',
 *   success: export function (res) {
 *     console.log(res.width)
 *     console.log(res.height)
 *   }
 * })
 *
 * Taro.chooseImage({
 *   success: export function (res) {
 *     Taro.getImageInfo({
 *       src: res.tempFilePaths[0],
 *       success: export function (res) {
 *         console.log(res.width)
 *         console.log(res.height)
 *       }
 *     })
 *   }
 * })
 * ```
 */

exports.previewImage = previewImage;

var getImageInfoInternal = function getImageInfoInternal(_ref3) {
  var src = _ref3.src,
      success = _ref3.success,
      fail = _ref3.fail,
      complete = _ref3.complete;
  fetch(src).then(function (res) {
    return res.blob();
  }).then(function (blob) {
    var url = URL.createObjectURL(blob);
    var image = new Image();

    image.onload = function () {
      success === null || success === void 0 ? void 0 : success({
        width: image.naturalWidth,
        height: image.naturalHeight,
        path: url,
        orientation: 'up',
        type: blob.type
      });
      complete === null || complete === void 0 ? void 0 : complete();
    };

    image.onerror = function (e) {
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: e.message
      });
      complete === null || complete === void 0 ? void 0 : complete();
    };

    image.src = url;
  }).catch(function (err) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: err.message
    });
    complete === null || complete === void 0 ? void 0 : complete();
  });
};

var getImageInfo = _util.limited.async('getImageInfo', getImageInfoInternal);

exports.getImageInfo = getImageInfo;

var saveImageToPhotosAlbum = _util.unsupported.async('saveImageToPhotosAlbum');

exports.saveImageToPhotosAlbum = saveImageToPhotosAlbum;

var compressImage = _util.unsupported.async('compressImage');

exports.compressImage = compressImage;