"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createVideoContext = exports.chooseVideo = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _mpromisify = _interopRequireDefault(require("mpromisify"));

var _util = require("../_util");

var _dom = require("../_util/dom");

var chooseVideoInternal = function chooseVideoInternal(_ref) {
  var _ref$sourceType = _ref.sourceType,
      sourceType = _ref$sourceType === void 0 ? ['album', 'camera'] : _ref$sourceType,
      success = _ref.success,
      complete = _ref.complete;
  var el = (0, _dom.createInput)(sourceType);
  el.setAttribute('accept', 'video/*');

  el.onchange = function (e) {
    var target = e.target;
    var file = target.files ? Array.from(target.files)[0] : null;

    if (!file) {
      return;
    }

    var video = document.createElement('video');
    var blob = new Blob([file], {
      type: file.type
    });
    var url = URL.createObjectURL(blob); // 尝试获取视频的宽高信息

    video.onloadedmetadata = function () {
      var result = {
        tempFilePath: url,
        duration: video.duration,
        size: file.size,
        height: video.videoHeight,
        width: video.videoWidth
      };
      success === null || success === void 0 ? void 0 : success(result);
      complete === null || complete === void 0 ? void 0 : complete();
    };

    video.src = url;
  };

  var event = document.createEvent('MouseEvents');
  event.initEvent('click', true, true);
  el.dispatchEvent(event);
};

var chooseVideo = (0, _mpromisify.default)(_util.limited.async('chooseVideo', chooseVideoInternal));
exports.chooseVideo = chooseVideo;

var VideoContext = /*#__PURE__*/function () {
  function VideoContext(id) {
    (0, _classCallCheck2.default)(this, VideoContext);
    this.videoId = id;

    if (this.videoId) {
      this.videoEl = document.querySelector("#".concat(this.videoId));
    }
  }

  (0, _createClass2.default)(VideoContext, [{
    key: "play",
    value: function play() {
      if (this.videoEl) {
        this.videoEl.play();
      }
    }
  }, {
    key: "pause",
    value: function pause() {
      if (this.videoEl) {
        this.videoEl.pause();
      }
    }
  }, {
    key: "seek",
    value: function seek(position) {
      if (this.videoEl) {
        this.videoEl.currentTime = position;
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      if (this.videoEl) {
        this.videoEl.currentTime = 0;
        this.videoEl.pause();
      }
    }
  }, {
    key: "sendDanmu",
    value: function sendDanmu() {}
  }, {
    key: "playbackRate",
    value: function playbackRate(rate) {
      if (this.videoEl) {
        this.videoEl.playbackRate = rate;
      }
    }
  }, {
    key: "requestFullScreen",
    value: function requestFullScreen() {
      if (this.videoEl) {
        this.videoEl.requestFullscreen();
      }
    }
  }, {
    key: "exitFullScreen",
    value: function exitFullScreen() {
      document.exitFullscreen();
    }
  }, {
    key: "showStatusBar",
    value: function showStatusBar() {}
  }, {
    key: "hideStatusBar",
    value: function hideStatusBar() {}
  }]);
  return VideoContext;
}();

var createVideoContextInternal = function createVideoContextInternal(id) {
  return new VideoContext(id);
};

var createVideoContext = _util.limited.never('createVideoContext', createVideoContextInternal);

exports.createVideoContext = createVideoContext;