"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = request;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _jsonpRetry = _interopRequireDefault(require("jsonp-retry"));

var _util = require("../_util");

// @ts-ignore
function generateRequestUrlWithParams(url, params) {
  params = typeof params === 'string' ? params : (0, _util.serializeParams)(params);

  if (params) {
    url += (~url.indexOf('?') ? '&' : '?') + params;
  }

  return url.replace('?&', '?');
}

function request(options) {
  options = options || {};

  if (typeof options === 'string') {
    options = {
      url: options
    };
  }

  var _options = options,
      success = _options.success,
      complete = _options.complete,
      fail = _options.fail;
  var url = options.url;
  var params = {};
  var res = {};

  if (options.jsonp) {
    Object.assign(params, options);
    params.params = options.data;
    params.cache = options.jsonpCache;

    if (typeof options.jsonp === 'string') {
      params.name = options.jsonp;
    }

    delete params.jsonp;
    return (0, _jsonpRetry.default)(url, params).then(function (data) {
      res.statusCode = 200;
      res.data = data;
      typeof success === 'function' && success(res);
      typeof complete === 'function' && complete(res);
      return res;
    }).catch(function (err) {
      typeof fail === 'function' && fail(err);
      typeof complete === 'function' && complete(res);
      return Promise.reject(err);
    });
  }

  params.method = options.method || 'GET';
  var methodUpper = params.method.toUpperCase();
  params.cache = options.cache || 'default';

  if (methodUpper === 'GET' || methodUpper === 'HEAD') {
    url = generateRequestUrlWithParams(url, options.data);
  } else if ((0, _typeof2.default)(options.data) === 'object') {
    options.header = options.header || {};
    var keyOfContentType = Object.keys(options.header).find(function (item) {
      return item.toLowerCase() === 'content-type';
    });

    if (!keyOfContentType) {
      options.header['Content-Type'] = 'application/json';
    }

    var contentType = options.header[keyOfContentType || 'Content-Type'];

    if (contentType.indexOf('application/json') >= 0) {
      params.body = JSON.stringify(options.data);
    } else if (contentType.indexOf('application/x-www-form-urlencoded') >= 0) {
      params.body = (0, _util.serializeParams)(options.data);
    } else {
      params.body = options.data;
    }
  } else {
    params.body = options.data;
  }

  if (options.header) {
    params.headers = options.header;
  }

  if (options.mode) {
    params.mode = options.mode;
  }

  if (options.signal) {
    params.signal = options.signal;
  }

  params.credentials = options.credentials;
  return fetch(url, params).then(function (response) {
    if (!response) {
      var errorResponse = {
        ok: false
      };
      throw errorResponse;
    }

    res.statusCode = response.status;
    res.header = {};

    for (var _i = 0, _Object$keys = Object.keys(response.headers); _i < _Object$keys.length; _i++) {
      var key = _Object$keys[_i];
      res.header[key] = response.headers.get(key);
    }

    if (!response.ok) {
      throw response;
    }

    if (options.responseType === 'arraybuffer') {
      return response.arrayBuffer();
    }

    if (res.statusCode !== 204) {
      if (options.dataType === 'json' || typeof options.dataType === 'undefined') {
        return response.json();
      }
    }

    if (options.responseType === 'text' || options.dataType === 'text') {
      return response.text();
    }

    return Promise.resolve(null);
  }).then(function (data) {
    res.data = data;
    typeof success === 'function' && success(res);
    typeof complete === 'function' && complete(res);
    return res;
  }).catch(function (err) {
    typeof fail === 'function' && fail(err);
    typeof complete === 'function' && complete(res);
    return Promise.reject(err);
  });
}