"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uploadFile = exports.NETWORK_TIMEOUT = void 0;

var _handler = require("../_util/handler");

var NETWORK_TIMEOUT = 60000;
exports.NETWORK_TIMEOUT = NETWORK_TIMEOUT;

var url2Blob = function url2Blob(url) {
  return new Promise(function (resolve, reject) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, true);
    xhr.responseType = 'blob';

    xhr.onload = function () {
      if (this.status === 200) {
        resolve(this.response);
      } else {
        reject({
          status: this.status
        });
      }
    };

    xhr.send();
  });
};
/**
 * 将本地资源上传到服务器。客户端发起一个 HTTPS POST 请求，其中 content-type 为 multipart/form-data。使用前请注意阅读相关说明。
 */


var uploadFile = function uploadFile(_ref) {
  var url = _ref.url,
      filePath = _ref.filePath,
      name = _ref.name,
      header = _ref.header,
      formData = _ref.formData,
      success = _ref.success,
      fail = _ref.fail,
      complete = _ref.complete;
  var timeoutTimer = null;
  var formKey;
  var xhr = new XMLHttpRequest();
  var form = new FormData();
  var callbackManager = {
    headersReceived: new _handler.CallbackManager(),
    progressUpdate: new _handler.CallbackManager()
  };
  xhr.open('POST', url);

  for (var key in header) {
    xhr.setRequestHeader(key, header[key]);
  }

  for (formKey in formData) {
    form.append(formKey, formData[formKey]);
  }

  xhr.upload.onprogress = function (_ref2) {
    var loaded = _ref2.loaded,
        total = _ref2.total;
    callbackManager.progressUpdate.trigger({
      progress: Math.round(loaded / total * 100),
      totalBytesSent: loaded,
      totalBytesExpectedToSent: total
    });
  };

  xhr.onreadystatechange = function () {
    if (xhr.readyState !== 2) return;
    callbackManager.headersReceived.trigger({
      header: xhr.getAllResponseHeaders()
    });
  };

  xhr.onload = function () {
    var status = xhr.status;

    if (timeoutTimer) {
      clearTimeout(timeoutTimer);
    }

    success === null || success === void 0 ? void 0 : success({
      statusCode: status,
      data: xhr.responseText || xhr.response
    });
    complete === null || complete === void 0 ? void 0 : complete();
  };

  xhr.onabort = function () {
    if (timeoutTimer) {
      clearTimeout(timeoutTimer);
    }

    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: 'uploadFile:fail abort'
    });
    complete === null || complete === void 0 ? void 0 : complete();
  };

  xhr.onerror = function (e) {
    if (timeoutTimer) {
      clearTimeout(timeoutTimer);
    }

    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: "uploadFile:fail ".concat(e.message)
    });
    complete === null || complete === void 0 ? void 0 : complete();
  };
  /**
   * 中断任务
   */


  var abort = function abort() {
    if (timeoutTimer) {
      clearTimeout(timeoutTimer);
    }

    xhr.abort();
  };

  var send = function send() {
    xhr.send(form);
    timeoutTimer = setTimeout(function () {
      xhr.onabort = null;
      xhr.onload = null;
      xhr.upload.onprogress = null;
      xhr.onreadystatechange = null;
      xhr.onerror = null;
      abort();
      fail === null || fail === void 0 ? void 0 : fail({
        errMsg: 'uploadFile:fail timeout'
      });
      complete === null || complete === void 0 ? void 0 : complete();
    }, NETWORK_TIMEOUT);
  };

  url2Blob(filePath).then(function (fileObj) {
    form.append(name, fileObj, fileObj.name || "file-".concat(Date.now()));
    send();
  }).catch(function (e) {
    fail === null || fail === void 0 ? void 0 : fail({
      errMsg: "uploadFile:fail ".concat(e.message)
    });
    complete === null || complete === void 0 ? void 0 : complete();
  });
  /**
   * 监听 HTTP Response Header 事件。会比请求完成事件更早
   * @param {HeadersReceivedCallback} callback HTTP Response Header 事件的回调函数
   */

  var onHeadersReceived = callbackManager.headersReceived.add;
  /**
   * 取消监听 HTTP Response Header 事件
   * @param {HeadersReceivedCallback} callback HTTP Response Header 事件的回调函数
   */

  var offHeadersReceived = callbackManager.headersReceived.remove;
  /**
   * 监听进度变化事件
   * @param {ProgressUpdateCallback} callback HTTP Response Header 事件的回调函数
   */

  var onProgressUpdate = callbackManager.progressUpdate.add;
  /**
   * 取消监听进度变化事件
   * @param {ProgressUpdateCallback} callback HTTP Response Header 事件的回调函数
   */

  var offProgressUpdate = callbackManager.progressUpdate.remove;
  return {
    abort: abort,
    onHeadersReceived: onHeadersReceived,
    offHeadersReceived: offHeadersReceived,
    onProgressUpdate: onProgressUpdate,
    offProgressUpdate: offProgressUpdate
  };
};

exports.uploadFile = uploadFile;